# Zach Gildersleeve
# zgildersleeve@gmail.com
# CS6630 - Scientific Visualization
# Project 1
# Sept. 26, 2006
#
# contour.tcl - This script creates a GUI powered tool to generate
# contours for three datasets.  The first two, body.vtk and brain.vtk
# are vtkStructuredPoints that are piped into the vtkContourFilter.
#
# The third dataset is a DEM file that is read using the built-in
# vtk reader for that data, then written to a temp vtkStructuredPoints
# file before being read using the above pipeline.
#
# The GUI allows the user to switch between the three datasets,
# and allows the user to interactively manipulate the contour settings
# (number of contour lines, min and max contour value).


catch {load vtktcl}
set VTK_TCL ".."
set VTK_DATA "../vtkdata"
set TOPO_OUTPUT "topoStructuredPoints.vtk"

set scaleMin -5000
set scaleMax 5000

vtkStructuredPointsReader body
	body SetFileName $VTK_DATA/body.vtk
	body Update

vtkStructuredPointsReader brain
	brain SetFileName $VTK_DATA/brain.vtk
	brain Update

vtkDEMReader dem
	dem SetFileName topo.dem
	dem Update
set demElevationBounds [dem GetElevationBounds]

vtkStructuredPointsWriter writer
	writer SetFileName $TOPO_OUTPUT
	writer SetInput [dem GetOutput]
	writer Write

vtkStructuredPointsReader topo
	topo SetFileName $TOPO_OUTPUT
	topo Update

vtkContourFilter contour
	contour SetInput [body GetOutput]
	contour GenerateValues 20 20 200

vtkPolyDataMapper mapper
	mapper SetInput [contour GetOutput]
	mapper ScalarVisibilityOn
	mapper SetScalarRange 0 255

vtkActor actor
	actor SetMapper mapper

vtkRenderer ren1
vtkRenderWindow renWin
	renWin AddRenderer ren1
vtkRenderWindowInteractor iren
	iren SetRenderWindow renWin

vtkInteractorStyleTrackballCamera style
	iren SetInteractorStyle style

ren1 AddActor actor
ren1 SetBackground 0.5 0.5 0.5

renWin SetSize 500 500

label .l4 -text "Select Data Source"
radiobutton .r1 -text "body.vtk" -command setImage -var image -value body
radiobutton .r2 -text "brain.vtk" -command setImage -var image -value brain
radiobutton .r3 -text "topo.dem" -command setImage -var image -value topo

label .l5 -text "Select Color Scale"
radiobutton .r4 -text "rainbow" -command setColor -var color -value rainbow
radiobutton .r5 -text "white  " -command setColor -var color -value mono

label .l1 -text "Number of Contour Lines"
scale .s1 -from 5 -to 50 -length 200 -resolution 5 -showvalue true -orient horizontal -command setContour
label .l2 -text "Minimum Line Value"
scale .s2 -from $scaleMin -to $scaleMax -length 200 -resolution 5 -showvalue true -orient horizontal -command setContour
label .l3 -text "Maximum Line Value"
scale .s3 -from $scaleMin -to $scaleMax -length 200 -resolution 5 -showvalue true -orient horizontal -command setContour


# defaults for GUI upon startup
.s1 set 25
.s2 set 0
.s3 set 255
set image body
set color rainbow

button .save -text "Save 'frame000.ppm'" -command saveFrame
button .view -text "Save View" -command saveView
button .reset -text "Reset Camera" -command {reset}
button .quit -text "Quit" -command {exit}

grid .l1 -row 0 -column 0
grid .s1 -row 0 -column 1
grid .l2 -row 1 -column 0
grid .s2 -row 1 -column 1
grid .l3 -row 2 -column 0
grid .s3 -row 2 -column 1

grid .l4 -row 4 -column 0
grid .r1 -row 5 -column 0
grid .r2 -row 6 -column 0
grid .r3 -row 7 -column 0

grid .l5 -row 4 -column 1
grid .r4 -row 5 -column 1
grid .r5 -row 6 -column 1

grid .reset -row 8 -column 1
grid .quit -row 9 -column 1

renWin Render

###
# sets the parameters for the contour filter
#
proc setContour {args} {
	global numberLines
	global minValue
	global maxValue

	set numberLines [.s1 get]
	set minValue [.s2 get]
	set maxValue [.s3 get]

	contour GenerateValues $numberLines $minValue $maxValue
	renWin Render
}

###
# switches between the three datasets,
# and sets the appropriate slider ranges
#
proc setImage {args} {
	global image
	global demElevationBounds
	global scaleMin
	global scaleMax
	global ren1
	switch $image {
		"body" 	{contour SetInput [body GetOutput]
			.s2 set 0
			.s3 set 255
			set scaleMin 0
			set scaleMax 500
			mapper SetScalarRange 0 255
			reset
			}
		"brain" {contour SetInput [brain GetOutput]
			.s2 set 0
			.s3 set 255
			set scaleMin 0
			set scaleMax 500
			mapper SetScalarRange 0 255
			reset
			}
		"topo"	{contour SetInput [topo GetOutput]
			.s2 set [lindex $demElevationBounds 0]
			.s3 set [lindex $demElevationBounds 1]
			set scaleMax [lindex $demElevationBounds 0]
			set scaleMax [lindex $demElevationBounds 1]
			mapper SetScalarRange [lindex $demElevationBounds 0] [lindex $demElevationBounds 1]
			reset
			}
	}
}

###
# switches between the rainbow and white
# color mapping
#
proc setColor {args} {
	global color
	switch $color {
		"rainbow"	{mapper ScalarVisibilityOn
				mapper SetScalarRange 0 255
				renWin Render
				}
		"mono"		{mapper ScalarVisibilityOff
				renWin Render
				}
	}
}

###
# resets the camera. mimics pressing the 'r' key 
#
proc reset {args} {
	[ren1 GetActiveCamera] OrthogonalizeViewUp
	ren1 ResetCamera
	ren1 ResetCameraClippingRange
	[ren1 GetActiveCamera] SetViewUp 0 1 0
	renWin Render
}